<?php
/*
Plugin Name: Twitpost
Plugin URI:  http://blog.indeedle.com/twitpost-119/
Description: A plugin that notifies your twitter feed when you've made a new post in your blog.
Author: C. Finegan
Version: 0.0.2
Author URI: http://www.indeedle.com/
*/

/*  Copyright 2009  C. Finegan  (email : contact@indeedle.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// Grab the twitter file
require_once('twitterAPI.class.php');

// Add to the menu
add_action('admin_menu', 'twitpost_addmenu');
add_filter('wp_insert_post', 'twitpost_filterpublish', 5, 1);
register_activation_hook( __FILE__, 'twitpost_install' );
if (function_exists('register_uninstall_hook'))
	register_uninstall_hook(__FILE__, 'twitpost_uninstall');

function twitpost_addmenu()
{
	add_options_page('Twitpost', 'Twitpost', 'manage_options', __FILE__, 'twitpost_options_page');
}

// The filter
function twitpost_filterpublish($postID)
{
	// Get the post
	$post = get_post($postID);
	
	// Let's check, if the post isn't set as published then we're not going to do anything
	// Only do anything if it's both publish and we're authenticated
	
	// Secondly we need to work out, is this post being skipped?
	$status = get_post_meta($postID, 'twitpost_action', true);

	if(($post->post_status == "publish" || $status == 'post') && get_option('twitpost_authenticated') == 1)
	{
		// We also need to get what the current option is
		$method = get_option('twitpost_method');
		
		$twitdo = false; // Whether or not we should twitter
		
		if($method == 'man')
		{
			// To add the twitter the post needs to have a value of 'post' in the custom field
			if($status == 'post' && $status != 'added')
				$twitdo = true;
		}
		elseif($method == 'auto')
		{
			// If the post has a skip value OR has a added value, then we don't do anything
			if($status == 'skip' || $status == 'added')
				$twitdo = false;
			else
				$twitdo = true;
		}

		if($twitdo)
		{		
			// Firstly, check the validation
			$password = get_option('twitpost_password');
			$username = get_option('twitpost_username');
			
			$twit = new twitterAPI($username, $password);
			
			$valid = $twit->validateLogin();
			
			if($twit->validateLogin() == 1)
			{	
					
				// Update the custom field to posted
				update_post_meta($postID, 'twitpost_action', 'added', $status);
				
				// Add the twitter (for now we'll just email as a test)
				$message = get_option('twitpost_message');
				
				// Get the variables to replace the message with
				global $current_user;
				get_currentuserinfo();
				
				$bloginfo = get_bloginfo();
	
				$user = $current_user->display_name;
				$title = $post->post_title;
				$url = $post->guid;
				$site = get_bloginfo('siteurl');
				$name = get_bloginfo('name');
				
				$twitMessage = str_replace(array('USER','TITLE','URL', 'SITE', 'NAME'), array($user, $title, $url, $site, $name), $message);
				
				
				if($password != '' && $username != '')
				{
					$twit->addStatus($twitMessage);
				}
			}
			else 
			{
				// We couldn't update the status
				update_option('twitpost_authenticated', 2);
			}
		}
	}
	
	return $postID;
}

// Installs the plugin
function twitpost_install()
{
	// Add the options to the database
	add_option('twitpost_password', '');
	add_option('twitpost_username', '');
	add_option('twitpost_authenticated', '0');
	add_option('twitpost_message', 'USER has blogged about "TITLE" at URL');
	add_option('twitpost_method', 'auto');
}

function twitpost_uninstall()
{
	delete_option('twitpost_password');
	delete_option('twitpost_username');
	delete_option('twitpost_authenticated');
	delete_option('twitpost_message');
	delete_option('twitpost_method');
}

// The actual options page
function twitpost_options_page()
{
	$password = get_option('twitpost_password');
	$username = get_option('twitpost_username');
	$authenticated = get_option('twitpost_authenticated');
	$message = get_option('twitpost_message');
	
	$message = (empty($message)) ? 'USER has blogged about "TITLE" at URL' : $message;
	
	$twit = new twitterAPI($username, $password);
	
	$valid = 0;
	$err = array(); // Hold the error messages
	
	if($password != '' && $username != '' && $authenticated == 0)
	{	
		// Validate login
		$valid = $twit->validateLogin();
		if($valid == 1)
		{
			// Basically by doing this we don't reauthenticate every page load
			// instead we only do it when the username/password was resubmitted
			update_option('twitpost_authenticated', 1);
			$authenticated = 1;
		}
		elseif($valid == 2)
		{
			$err[] = 'Couldn\'t authenticate you with Twitter. Please check that your username &amp; passsword are both correct.';
		}
	}
	elseif($authenticated == 2)
	{
		// This means during the script validation failed, this can be caused by a user's password being changed
		$err[] = 'Can no longer connect to Twitter, did you change your password? Please update your login details.';
	}
	
	?><div class="wrap">
<h2>Twitpost</h2>
<p>Twitpost will add a short note to your Twitter feed when you add a new post. This can be called automatically, or it can be called manually.</p>

<h3>Authentication</h3>
<p>This is needed to update your twitter feed.</p>
<?php if(count($err) > 0){ ?><p style="padding: .5em; font-weight: bold;" class="error"><?php echo join('<br />', $err); ?></p><? } elseif($valid == 1 && $authenticated == 1){ ?><p style="padding: .5em; font-weight: bold;" class="updated fade">Great! Authenticated with twitter :)</p><? } ?>
<form method="post" action="options.php">
<?php wp_nonce_field('update-options'); ?>
<table class="form-table">
<tr valign="top">
<th scope="row">Twitter Username:</th>
<td><input type="text" name="twitpost_username" value="<?php echo strtolower(str_replace(array("\n", "\r", "\t", " ", "\o", "\xOB"), '', $username)); ?>" class="code" />
<span class="setting-description">Your twitter username.</span></td>
</tr><tr valign="top">
<th scope="row">Twitter Password:</th>
<td><input type="password" name="twitpost_password" value="<?php echo $password; ?>" class="code" />
<span class="setting-description">Your twitter password.</span></td>
</tr>
</table>
<input type="hidden" name="action" value="update" />
<input type="hidden" name="twitpost_authenticated" value="0" />
<input type="hidden" name="page_options" value="twitpost_password,twitpost_username,twitpost_authenticated" />
<p class="submit">
<input type="submit" name="Submit" value="<?php _e('Update Login') ?>" />
</p>
</form>
<?php if($authenticated > 0){
// The login is valid, addional options to set
?>
<h3>Settings</h3>
<p>If you set the method to automatic, Twitpost will add the message to your twitter feed for every post you make, unless you specifically exclude the post.</p>
<p>With manual Twitpost will only add the message to your twitter feed if you specifically tell it to.</p>
<p>Add a custom field to your post titled <code>twitpost_action</code>. A value of <code>post</code> when method is manual means the post will be sent to twitter, while a value of <code>skip</code> when the method is automatic means the post you've written will not be sent to twitter.</p>
<p>Once the written post has been published, <code>twitpost_action</code> will be changed to <code>added</code> which means if edited in the future it will not be sent to twitter. If you want to notify twitter of it being updated, just replace <code>added</code> with <code>post</code> when editing the post, and twitter will be notified again.</p>
<form method="post" action="options.php">
<?php wp_nonce_field('update-options'); ?>
<table class="form-table">
<tr valign="top">
<th scope="row">Method:</th>
<td><select name="twitpost_method" class="code">
		<option class="code" value="auto"<?php if(get_option('twitpost_method') == 'auto') { echo " selected"; } ?>>Automatic (Upon publish)</option>
		<option class="code" value="man"<?php if(get_option('twitpost_method') == 'man') { echo " selected"; } ?>>Manual (Post by post)</option>
		</select><br />
<span class="setting-description"></span></td>
</tr>
<tr valign="top">
<th scope="row">Message:</th>
<td><textarea name="twitpost_message" class="large-text code"><?php echo $message; ?></textarea>
<span class="setting-description">This is the message that will be posted on twitter. <code>USER</code>, <code>TITLE</code>, <code>URL</code>, <code>SITE</code> and <code>NAME</code> will be replaced with your name, post title, post link, blog link and blog title respectivly.</span></td>
</tr>

</table>
<input type="hidden" name="action" value="update" />
<input type="hidden" name="page_options" value="twitpost_method,twitpost_message" />
<p class="submit">
<input type="submit" name="Submit" value="<?php _e('Save Settings') ?>" />
</p>
</form>
<? } ?>
</div>
<? }

?>
